#!/usr/bin/env python3
import hashlib
import os


def md5(fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


class TestAppProtocol:
    def test_protocol_data_import(self):
        from blueye.legacyprotocol import protocol_data # noqa: F401

    def test_protocol_data_hash(self, request):
        from blueye.legacyprotocol import _generator_hash, _json_hash

        root_dir_path = request.fspath.dirname + "/../../"
        assert _json_hash == md5(
            os.path.join(root_dir_path, "ProtocolDefinitions", "udp_protocol.json")
        )
        assert _generator_hash == md5(
            os.path.join(root_dir_path, "generators", "generate_udp_protocol.py")
        )



# This file is autogenerated. Please do not edit
import struct
from .tcp_client import TcpClientBase

class TcpClientV1(TcpClientBase):
    protocol_version = 1
    def motion_input(self, surge_motion_input, sway_motion_input, heave_motion_input, yaw_motion_input, slow_input, boost_input):
        """Send a motion_input command over TCP

        Args:
            surge_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            sway_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            heave_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            yaw_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            slow_input (numpy data type:<f4): valid range is <0, 1> 
            boost_input (numpy data type:<f4): valid range is <0, 1> 
        """
        if not -1 <= surge_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for surge_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=surge_motion_input))

        if not -1 <= sway_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for sway_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=sway_motion_input))

        if not -1 <= heave_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for heave_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=heave_motion_input))

        if not -1 <= yaw_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for yaw_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=yaw_motion_input))

        if not 0 <= slow_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for slow_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=1) +
                " but got value: {name}".format(name=slow_input))

        if not 0 <= boost_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for boost_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=1) +
                " but got value: {name}".format(name=boost_input))

        command_identifier = b'j'
        msg = command_identifier
        msg += struct.pack('ffffff', surge_motion_input, sway_motion_input, heave_motion_input, yaw_motion_input, slow_input, boost_input)
        self.send_and_receive(msg, expects_reply=False)

    def set_lights(self, brightness_upper, brightness_lower):
        """Send a set_lights command over TCP

        Args:
            brightness_upper (numpy data type:<u1): valid range is <0, 255> 
            brightness_lower (numpy data type:<u1): valid range is <0, 255> 
        """
        if not 0 <= brightness_upper <= 255:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for brightness_upper is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=255) +
                " but got value: {name}".format(name=brightness_upper))

        if not 0 <= brightness_lower <= 255:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for brightness_lower is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=255) +
                " but got value: {name}".format(name=brightness_lower))

        command_identifier = b'l'
        msg = command_identifier
        msg += struct.pack('BB', brightness_upper, brightness_lower)
        self.send_and_receive(msg, expects_reply=False)

    def ping(self):
        """Send a ping command over TCP
        """
        command_identifier = b'p'
        msg = command_identifier
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'P')

    def start_recording(self):
        """Send a start_recording command over TCP
        """
        command_identifier = b'r'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def stop_recording(self):
        """Send a stop_recording command over TCP
        """
        command_identifier = b'R'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def start_compass_calibration(self):
        """Send a start_compass_calibration command over TCP
        """
        command_identifier = b'i'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def cancel_compass_calibration(self):
        """Send a cancel_compass_calibration command over TCP
        """
        command_identifier = b'I'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def save_compass_calibration(self):
        """Send a save_compass_calibration command over TCP
        """
        command_identifier = b'c'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def user_geo_location(self, longitude, latitude):
        """Send a user_geo_location command over TCP

        Args:
            longitude (numpy data type:<f8): longitude that is included in the log file in degrees
            latitude (numpy data type:<f8): latitude that is included in the log file in degrees
        """
        command_identifier = b'g'
        msg = command_identifier
        msg += struct.pack('dd', longitude, latitude)
        self.send_and_receive(msg, expects_reply=False)

    def watchdog(self, connection_duration):
        """Send a watchdog command over TCP

        Args:
            connection_duration (numpy data type:<i2): time in seconds since connected to drone
        """
        command_identifier = b'w'
        msg = command_identifier
        msg += struct.pack('h', connection_duration)
        self.send_and_receive(msg, expects_reply=False)

    def auto_heading_on(self):
        """Send a auto_heading_on command over TCP
        """
        command_identifier = b'h'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_heading_off(self):
        """Send a auto_heading_off command over TCP
        """
        command_identifier = b'H'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_depth_on(self):
        """Send a auto_depth_on command over TCP
        """
        command_identifier = b'd'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_depth_off(self):
        """Send a auto_depth_off command over TCP
        """
        command_identifier = b'D'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def set_system_time(self, unix_timestamp):
        """Send a set_system_time command over TCP

        set the system time on the on-board computer

        Args:
            unix_timestamp (numpy data type:<i4): 
        """
        command_identifier = b't'
        msg = command_identifier
        msg += struct.pack('i', unix_timestamp)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_exposure(self, exposure_value):
        """Send a set_camera_exposure command over TCP

        Args:
            exposure_value (numpy data type:<i4): 1 = 1/1000th of a second, 5 = 1/200th of a second. Valid values are in the range <1, 5000>
        """
        command_identifier = b've'
        msg = command_identifier
        msg += struct.pack('i', exposure_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_whitebalance(self, whitebalance_value):
        """Send a set_camera_whitebalance command over TCP

        Args:
            whitebalance_value (numpy data type:<i4): valid values are in the range <2800, 9300>
        """
        command_identifier = b'vw'
        msg = command_identifier
        msg += struct.pack('i', whitebalance_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_hue(self, hue_value):
        """Send a set_camera_hue command over TCP

        Args:
            hue_value (numpy data type:<i4): valid values are in the range <-40, 40>
        """
        command_identifier = b'vh'
        msg = command_identifier
        msg += struct.pack('i', hue_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_bitrate(self, bitrate_value):
        """Send a set_camera_bitrate command over TCP

        Args:
            bitrate_value (numpy data type:<i4): set camera bitrate in bits. Valid values are in range <1 000 000, 16 000 000> 
        """
        command_identifier = b'vb'
        msg = command_identifier
        msg += struct.pack('i', bitrate_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def get_camera_parameters(self):
        """Send a get_camera_parameters command over TCP

        Returns:
            parameter (numtyp data type:<u1)
            camera_bitrate (numtyp data type:<i4)
            camera_exposure (numtyp data type:<i4)
            camera_whitebalance (numtyp data type:<i4)
            camera_hue (numtyp data type:<i4)
        """
        command_identifier = b'Va'
        msg = command_identifier
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=17)
        return struct.unpack('<Biiii', reply)


class TcpClientV2(TcpClientBase):
    protocol_version = 2
    def motion_input(self, surge_motion_input, sway_motion_input, heave_motion_input, yaw_motion_input, slow_input, boost_input):
        """Send a motion_input command over TCP

        Args:
            surge_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            sway_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            heave_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            yaw_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            slow_input (numpy data type:<f4): valid range is <0, 1> 
            boost_input (numpy data type:<f4): valid range is <0, 1> 
        """
        if not -1 <= surge_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for surge_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=surge_motion_input))

        if not -1 <= sway_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for sway_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=sway_motion_input))

        if not -1 <= heave_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for heave_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=heave_motion_input))

        if not -1 <= yaw_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for yaw_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=yaw_motion_input))

        if not 0 <= slow_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for slow_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=1) +
                " but got value: {name}".format(name=slow_input))

        if not 0 <= boost_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for boost_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=1) +
                " but got value: {name}".format(name=boost_input))

        command_identifier = b'j'
        msg = command_identifier
        msg += struct.pack('ffffff', surge_motion_input, sway_motion_input, heave_motion_input, yaw_motion_input, slow_input, boost_input)
        self.send_and_receive(msg, expects_reply=False)

    def motion_input_tilt(self, surge_motion_input, sway_motion_input, heave_motion_input, yaw_motion_input, slow_input, boost_input, tilt_speed_input):
        """Send a motion_input_tilt command over TCP

        Args:
            surge_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            sway_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            heave_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            yaw_motion_input (numpy data type:<f4): valid range is <-1, 1> 
            slow_input (numpy data type:<f4): valid range is <0, 1> 
            boost_input (numpy data type:<f4): valid range is <0, 1> 
            tilt_speed_input (numpy data type:<f4): valid range is <-1, 1> Speed input for the camera tilt angle. 1 for max up, -1 for max down, 0 for stop.
        """
        if not -1 <= surge_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for surge_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=surge_motion_input))

        if not -1 <= sway_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for sway_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=sway_motion_input))

        if not -1 <= heave_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for heave_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=heave_motion_input))

        if not -1 <= yaw_motion_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for yaw_motion_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=yaw_motion_input))

        if not 0 <= slow_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for slow_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=1) +
                " but got value: {name}".format(name=slow_input))

        if not 0 <= boost_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for boost_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=1) +
                " but got value: {name}".format(name=boost_input))

        if not -1 <= tilt_speed_input <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for tilt_speed_input is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=tilt_speed_input))

        command_identifier = b'J'
        msg = command_identifier
        msg += struct.pack('fffffff', surge_motion_input, sway_motion_input, heave_motion_input, yaw_motion_input, slow_input, boost_input, tilt_speed_input)
        self.send_and_receive(msg, expects_reply=False)

    def set_lights(self, brightness_upper, brightness_lower):
        """Send a set_lights command over TCP

        Args:
            brightness_upper (numpy data type:<u1): valid range is <0, 255> 
            brightness_lower (numpy data type:<u1): valid range is <0, 255> 
        """
        if not 0 <= brightness_upper <= 255:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for brightness_upper is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=255) +
                " but got value: {name}".format(name=brightness_upper))

        if not 0 <= brightness_lower <= 255:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for brightness_lower is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=0, upper_limit=255) +
                " but got value: {name}".format(name=brightness_lower))

        command_identifier = b'l'
        msg = command_identifier
        msg += struct.pack('BB', brightness_upper, brightness_lower)
        self.send_and_receive(msg, expects_reply=False)

    def ping(self):
        """Send a ping command over TCP
        """
        command_identifier = b'p'
        msg = command_identifier
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'P')

    def start_recording(self):
        """Send a start_recording command over TCP
        """
        command_identifier = b'r'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def stop_recording(self):
        """Send a stop_recording command over TCP
        """
        command_identifier = b'R'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def set_overlay_temperature_enabled(self, overlay_temperature_enabled):
        """Send a set_overlay_temperature_enabled command over TCP

        Args:
            overlay_temperature_enabled (numpy data type:<u1): 1 is true, 0 is false
        """
        command_identifier = b'ot'
        msg = command_identifier
        msg += struct.pack('B', overlay_temperature_enabled)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_depth_enabled(self, overlay_depth_enabled):
        """Send a set_overlay_depth_enabled command over TCP

        Args:
            overlay_depth_enabled (numpy data type:<u1): 1 is true, 0 is false
        """
        command_identifier = b'od'
        msg = command_identifier
        msg += struct.pack('B', overlay_depth_enabled)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_heading_enabled(self, overlay_heading_enabled):
        """Send a set_overlay_heading_enabled command over TCP

        Args:
            overlay_heading_enabled (numpy data type:<u1): 1 is true, 0 is false
        """
        command_identifier = b'oh'
        msg = command_identifier
        msg += struct.pack('B', overlay_heading_enabled)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_tilt_enabled(self, overlay_tilt_enabled):
        """Send a set_overlay_tilt_enabled command over TCP

        Args:
            overlay_tilt_enabled (numpy data type:<u1): 1 is true, 0 is false
        """
        command_identifier = b'ol'
        msg = command_identifier
        msg += struct.pack('B', overlay_tilt_enabled)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_date_enabled(self, overlay_date_enabled):
        """Send a set_overlay_date_enabled command over TCP

        Args:
            overlay_date_enabled (numpy data type:<u1): 1 is true, 0 is false
        """
        command_identifier = b'oa'
        msg = command_identifier
        msg += struct.pack('B', overlay_date_enabled)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_logo_index(self, overlay_logo_index):
        """Send a set_overlay_logo_index command over TCP

        Args:
            overlay_logo_index (numpy data type:<i4): enum { NONE = 0, DEFAULT = 1, CUSTOM = 2 }
        """
        command_identifier = b'oo'
        msg = command_identifier
        msg += struct.pack('i', overlay_logo_index)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_depth_unit(self, overlay_depth_unit):
        """Send a set_overlay_depth_unit command over TCP

        Args:
            overlay_depth_unit (numpy data type:<i4): enum { METERS = 0, FEET = 1 }
        """
        command_identifier = b'oD'
        msg = command_identifier
        msg += struct.pack('i', overlay_depth_unit)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_temperature_unit(self, overlay_temperature_unit):
        """Send a set_overlay_temperature_unit command over TCP

        Args:
            overlay_temperature_unit (numpy data type:<i4): enum { CELSIUS = 0, FAHRENHEIT = 1 }
        """
        command_identifier = b'oT'
        msg = command_identifier
        msg += struct.pack('i', overlay_temperature_unit)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_tz_offset(self, overlay_tz_offset):
        """Send a set_overlay_tz_offset command over TCP

        Args:
            overlay_tz_offset (numpy data type:<i4): time zone offset in minutes
        """
        command_identifier = b'oz'
        msg = command_identifier
        msg += struct.pack('i', overlay_tz_offset)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_margin_width(self, overlay_margin_width):
        """Send a set_overlay_margin_width command over TCP

        Args:
            overlay_margin_width (numpy data type:<i4): margin width in pixels
        """
        command_identifier = b'ow'
        msg = command_identifier
        msg += struct.pack('i', overlay_margin_width)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_margin_height(self, overlay_margin_height):
        """Send a set_overlay_margin_height command over TCP

        Args:
            overlay_margin_height (numpy data type:<i4): margin height in pixels
        """
        command_identifier = b'oH'
        msg = command_identifier
        msg += struct.pack('i', overlay_margin_height)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_font_size(self, overlay_font_size):
        """Send a set_overlay_font_size command over TCP

        Args:
            overlay_font_size (numpy data type:<i4): choose from: { 15, 20, 25, 30, 35, 40 }
        """
        command_identifier = b'of'
        msg = command_identifier
        msg += struct.pack('i', overlay_font_size)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_title(self, overlay_title):
        """Send a set_overlay_title command over TCP

        Args:
            overlay_title (numpy data type:<u1[64]): Null terminated utf8 string padded to 64 characters. Only utf8 characters representable in latin1 will be rendered correctly.
        """
        command_identifier = b'oi'
        msg = command_identifier
        msg += struct.pack('64s', overlay_title)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_subtitle(self, overlay_subtitle):
        """Send a set_overlay_subtitle command over TCP

        Args:
            overlay_subtitle (numpy data type:<u1[64]): Null terminated utf8 string padded to 64 characters. Only utf8 characters representable in latin1 will be rendered correctly.
        """
        command_identifier = b'os'
        msg = command_identifier
        msg += struct.pack('64s', overlay_subtitle)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_overlay_date_format(self, overlay_date_format):
        """Send a set_overlay_date_format command over TCP

        Args:
            overlay_date_format (numpy data type:<u1[64]): Null terminated utf8 string padded to 64 characters. Only utf8 characters representable in latin1 will be rendered correctly.
        """
        command_identifier = b'oA'
        msg = command_identifier
        msg += struct.pack('64s', overlay_date_format)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def get_overlay_parameters(self):
        """Send a get_overlay_parameters command over TCP

        Returns:
            parameter (numtyp data type:<u1)
            temperature_enabled (numtyp data type:<u1)
            depth_enabled (numtyp data type:<u1)
            heading_enabled (numtyp data type:<u1)
            tilt_enabled (numtyp data type:<u1)
            date_enabled (numtyp data type:<u1)
            logo_index (numtyp data type:<i4)
            depth_unit (numtyp data type:<i4)
            temperature_unit (numtyp data type:<i4)
            tz_offset (numtyp data type:<i4)
            margin_width (numtyp data type:<i4)
            margin_height (numtyp data type:<i4)
            font_size (numtyp data type:<i4)
            title (numtyp data type:<u1[64])
            subtitle (numtyp data type:<u1[64])
            date_format (numtyp data type:<u1[64])
        """
        command_identifier = b'Oa'
        msg = command_identifier
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=226)
        return struct.unpack('<BBBBBBiiiiiii64s64s64s', reply)

    def start_compass_calibration(self):
        """Send a start_compass_calibration command over TCP
        """
        command_identifier = b'i'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def cancel_compass_calibration(self):
        """Send a cancel_compass_calibration command over TCP
        """
        command_identifier = b'I'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def save_compass_calibration(self):
        """Send a save_compass_calibration command over TCP
        """
        command_identifier = b'c'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def user_geo_location(self, longitude, latitude):
        """Send a user_geo_location command over TCP

        Args:
            longitude (numpy data type:<f8): longitude that is included in the log file in degrees
            latitude (numpy data type:<f8): latitude that is included in the log file in degrees
        """
        command_identifier = b'g'
        msg = command_identifier
        msg += struct.pack('dd', longitude, latitude)
        self.send_and_receive(msg, expects_reply=False)

    def watchdog(self, connection_duration):
        """Send a watchdog command over TCP

        Args:
            connection_duration (numpy data type:<i2): time in seconds since connected to drone
        """
        command_identifier = b'w'
        msg = command_identifier
        msg += struct.pack('h', connection_duration)
        self.send_and_receive(msg, expects_reply=False)

    def auto_heading_on(self):
        """Send a auto_heading_on command over TCP
        """
        command_identifier = b'h'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_heading_off(self):
        """Send a auto_heading_off command over TCP
        """
        command_identifier = b'H'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_depth_on(self):
        """Send a auto_depth_on command over TCP
        """
        command_identifier = b'd'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_depth_off(self):
        """Send a auto_depth_off command over TCP
        """
        command_identifier = b'D'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def auto_depth_step(self, direction):
        """Send a auto_depth_step command over TCP

        Args:
            direction (numpy data type:<i2):  1 for up, -1 for down
        """
        command_identifier = b'a'
        msg = command_identifier
        msg += struct.pack('h', direction)
        self.send_and_receive(msg, expects_reply=False)

    def auto_heading_step(self, direction):
        """Send a auto_heading_step command over TCP

        Args:
            direction (numpy data type:<i2): 1 for up, -1 for down
        """
        command_identifier = b'A'
        msg = command_identifier
        msg += struct.pack('h', direction)
        self.send_and_receive(msg, expects_reply=False)

    def set_system_time(self, unix_timestamp):
        """Send a set_system_time command over TCP

        set the system time on the on-board computer

        Args:
            unix_timestamp (numpy data type:<u4): 
        """
        command_identifier = b't'
        msg = command_identifier
        msg += struct.pack('I', unix_timestamp)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_water_density(self, water_density):
        """Send a set_water_density command over TCP

        Set the water density for improved accuracy of the depth sensor.

        Args:
            water_density (numpy data type:<i2): water density in grams per liter
        """
        command_identifier = b'W'
        msg = command_identifier
        msg += struct.pack('h', water_density)
        self.send_and_receive(msg, expects_reply=False)

    def take_still_picture(self):
        """Send a take_still_picture command over TCP

        Takes a still picture and stores it locally on the drone.
        """
        command_identifier = b's'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def toggle_tilt_stabilization(self):
        """Send a toggle_tilt_stabilization command over TCP

        Toggle active tilt stabilization on or off
        """
        command_identifier = b'S'
        msg = command_identifier
        self.send_and_receive(msg, expects_reply=False)

    def set_camera_exposure(self, exposure_value):
        """Send a set_camera_exposure command over TCP

        Args:
            exposure_value (numpy data type:<i4): 1 = 1/1000th of a second, 5 = 1/200th of a second. Valid values are in the range <1, 5000>
        """
        command_identifier = b've'
        msg = command_identifier
        msg += struct.pack('i', exposure_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_whitebalance(self, whitebalance_value):
        """Send a set_camera_whitebalance command over TCP

        Args:
            whitebalance_value (numpy data type:<i4): valid values are in the range <2800, 9300>
        """
        command_identifier = b'vw'
        msg = command_identifier
        msg += struct.pack('i', whitebalance_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_hue(self, hue_value):
        """Send a set_camera_hue command over TCP

        Args:
            hue_value (numpy data type:<i4): valid values are in the range <-40, 40>
        """
        command_identifier = b'vh'
        msg = command_identifier
        msg += struct.pack('i', hue_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_bitrate(self, bitrate_value):
        """Send a set_camera_bitrate command over TCP

        Args:
            bitrate_value (numpy data type:<i4): set camera bitrate in bits. Valid values are in range <1 000 000, 16 000 000> 
        """
        command_identifier = b'vb'
        msg = command_identifier
        msg += struct.pack('i', bitrate_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_framerate(self, framerate_value):
        """Send a set_camera_framerate command over TCP

        Args:
            framerate_value (numpy data type:<i4): valid values are 25 or 30
        """
        command_identifier = b'vf'
        msg = command_identifier
        msg += struct.pack('i', framerate_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def set_camera_resolution(self, resolution_value):
        """Send a set_camera_resolution command over TCP

        Args:
            resolution_value (numpy data type:<i4): valid values are 720 or 1080
        """
        command_identifier = b'vr'
        msg = command_identifier
        msg += struct.pack('i', resolution_value)
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=1)
        self.check_reply(reply, b'a')

    def get_camera_parameters(self):
        """Send a get_camera_parameters command over TCP

        Returns:
            parameter (numtyp data type:<u1)
            camera_bitrate (numtyp data type:<i4)
            camera_exposure (numtyp data type:<i4)
            camera_whitebalance (numtyp data type:<i4)
            camera_hue (numtyp data type:<i4)
            camera_resolution (numtyp data type:<i4)
            camera_framerate (numtyp data type:<i4)
        """
        command_identifier = b'Va'
        msg = command_identifier
        reply = self.send_and_receive(msg, expects_reply=True, receive_size=25)
        return struct.unpack('<Biiiiii', reply)

    def set_gripper_velocities(self, gripping_velocity, rotational_velocity):
        """Send a set_gripper_velocities command over TCP

        Args:
            gripping_velocity (numpy data type:<f4): valid range is <-1, 1> The opening/closing velocity of the gripper. Positive values for opening, negative for closing
            rotational_velocity (numpy data type:<f4): valid range is <-1, 1> The rotational velocity of the gripper. Positive values for clockwise, negative for counter-clockwise
        """
        if not -1 <= gripping_velocity <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for gripping_velocity is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=gripping_velocity))

        if not -1 <= rotational_velocity <= 1:
            raise ValueError(
                "Input argument out of range:" +
                " valid range for rotational_velocity is" +
                " <{lower_limit}, {upper_limit}>".format(lower_limit=-1, upper_limit=1) +
                " but got value: {name}".format(name=rotational_velocity))

        command_identifier = b'G'
        msg = command_identifier
        msg += struct.pack('ff', gripping_velocity, rotational_velocity)
        self.send_and_receive(msg, expects_reply=False)

    def set_thickness_gauge_sound_veloctiy(self, sound_velocity):
        """Send a set_thickness_gauge_sound_veloctiy command over TCP

        Args:
            sound_velocity (numpy data type:<i2): Sound velocity used for thickness calculation [m/s]
        """
        command_identifier = b'C'
        msg = command_identifier
        msg += struct.pack('h', sound_velocity)
        self.send_and_receive(msg, expects_reply=False)


class TcpClient:
    def __new__(self, protocol_version=None, *args, **kwargs):
        if protocol_version == None:
            return TcpClientV2(*args, **kwargs)
        elif protocol_version == 1:
            return TcpClientV1(*args, **kwargs)
        elif protocol_version == 2:
            return TcpClientV2(*args, **kwargs)
        else:
            raise ValueError(f"Unsupported protocol version: {protocol_version}")
            


#!/usr/bin/env python3

import logging
import platform
import socket
import time

from .exceptions import UnknownUDPPacketTypeError
from .udp_protocol_parser import AppProtocol


class UdpClient:
    def __init__(self, port=2010, protocol_description=None, logger=None, drone_ip="192.168.1.101"):
        self._port = port
        self._ip = "0.0.0.0"
        self._drone_ip = drone_ip

        if logger is None:
            # If no logger has been passed we'll use the module logger
            self.logger = logging.getLogger(__name__)
        else:
            self.logger = logger

        self._sock = None

        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        # To able to run multiple UdpClients on MacOS one needs to set
        # SO_REUSEPORT, however on Windows that option does not exist,
        # so we use SO_REUSEADDR for Windows and Linux.
        if platform.system() == "Darwin":
            self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        else:
            self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._ap = AppProtocol(protocol_description)
        self.bind()

    def __del__(self):
        self.logger.debug("Closing UDP socket")
        self._sock.close()

    def bind(self):
        self.logger.debug(f"Binding UDP socket to {self._ip}:{self._port}")
        self._sock.bind((self._ip, self._port))

    def _get_raw_data(self):
        while True:
            data_raw, addr = self._sock.recvfrom(1024)
            if addr[0] == self._drone_ip:
                return data_raw

    def get_data(self, packet_type=None, timeout=None):
        start_time = time.time()
        while timeout is None or time.time() < start_time + timeout:
            raw_data = self._get_raw_data()
            try:
                data = self._ap.unpack_data(raw_data)
            except UnknownUDPPacketTypeError:
                self.logger.warning("UDP packet type unknown. Please update the SDK")
                continue
            if packet_type is None or data[1] == packet_type:
                return data
        return None

    def get_data_dict(self, packet_type=None, timeout=None):
        start_time = time.time()
        while timeout is None or time.time() < start_time + timeout:
            raw_data = self._get_raw_data()
            try:
                data = self._ap.unpack_data_dict(raw_data)
            except UnknownUDPPacketTypeError:
                self.logger.warning("UDP packet type unknown. Please update the SDK")
                continue
            if packet_type is None or data["command_type"] == packet_type:
                return data
        return None



#!/usr/bin/env python3
import json
import struct
import pytest
from unittest.mock import Mock

from blueye.legacyprotocol import UdpClient

UDP_PORT = 32011

fake_json = json.loads(
    """
[
    {
        "version": "2",
        "messages": [
            {
                "name": "telemetry",
                "message_type": "1",
                "fields":[
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-v", "unit": ""},
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-t", "unit": ""},
                    {"description": "", "dtype": "<i1", "field_name": "i1-2", "unit": ""},
                    {"description": "", "dtype": "<u2", "field_name": "u2-2", "unit": ""},
                    {"description": "", "dtype": "<i2", "field_name": "i2-2", "unit": ""},
                    {"description": "", "dtype": "<u4", "field_name": "u4-2", "unit": ""},
                    {"description": "", "dtype": "<i4", "field_name": "i4-2", "unit": ""},
                    {"description": "", "dtype": "<u8", "field_name": "u8-2", "unit": ""},
                    {"description": "", "dtype": "<i8", "field_name": "i8-2", "unit": ""},
                    {"description": "", "dtype": "<f4", "field_name": "f4-2", "unit": ""},
                    {"description": "", "dtype": "<f8", "field_name": "f8-2-a", "unit": ""},
                    {"description": "", "dtype": "<f8", "field_name": "f8-2-b", "unit": ""}
                ]
            },
            {
                "name": "telemetry",
                "message_type": "2",
                "fields":[
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-v", "unit": ""},
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-t", "unit": ""},
                    {"description": "", "dtype": "<i1", "field_name": "i1-2", "unit": ""}
                ]
            },
            {
                "name": "telemetry",
                "message_type": "3",
                "fields":[
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-v", "unit": ""},
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-t", "unit": ""},
                    {"description": "", "dtype": "<i2", "field_name": "i1-2", "unit": ""}
                ]
            }
        ]
    }
]
"""
)


@pytest.fixture
def udp_client():
    client = UdpClient(
        protocol_description=fake_json, port=UDP_PORT, drone_ip="127.0.0.1"
    )
    client._sock = Mock()
    recvfrom_side_effect = [
        (struct.pack("<BBb", 2, 2, 2), ("127.0.0.1", UDP_PORT)),
        (struct.pack("<BBh", 2, 3, 3), ("127.0.0.1", UDP_PORT)),
    ]
    client._sock.recvfrom.side_effect = recvfrom_side_effect
    return client


def test_get_raw_data(udp_client):
    data = udp_client._get_raw_data()
    assert data == struct.pack("<BBb", 2, 2, 2)


def test_get_data(udp_client):
    data = udp_client.get_data()
    assert data == (2, 2, 2)


def test_get_data_t3(udp_client):
    data = udp_client.get_data(packet_type=3)
    assert data == (2, 3, 3)


def test_get_data_dict(udp_client):
    data = udp_client.get_data_dict()
    assert data == {"u1-2-v": 2, "u1-2-t": 2, "i1-2": 2}



from .tcp_protocol_class import TcpClient  # noqa F401
from .udp_client import UdpClient  # noqa F401
from .udp_protocol_dict import (  # noqa F401
    _generator_hash,
    _json_hash,
    protocol_data,
)
from .udp_protocol_parser import AppProtocol  # noqa F401

__all__ = [
    "TcpClient",
    "UdpClient",
    "_generator_hash",
    "_json_hash",
    "protocol_data",
    "AppProtocol",
]



#!/usr/bin/env python3
import gzip
import struct

import numpy as np

from .exceptions import UnknownUDPPacketTypeError, UnknownUDPVersionError
from .udp_protocol_dict import protocol_data


class AppProtocol:
    STRUCT_CONVERSION = {
        "<u1": "B",
        "<i1": "b",
        "<u2": "H",
        "<i2": "h",
        "<u4": "I",
        "<i4": "i",
        "<u8": "Q",
        "<i8": "q",
        "<f4": "f",
        "<f8": "d",
    }

    def __init__(self, protocol_definition=None):
        self._jdata = {}
        if protocol_definition is None:
            protocol_definition = protocol_data
        for protocol_version_data in protocol_definition:
            self._jdata[protocol_version_data["version"]] = {}
            for message in protocol_version_data["messages"]:
                self._jdata[protocol_version_data["version"]][message["message_type"]] = message[
                    "fields"
                ]
        self._last_version = sorted(self._jdata.keys())[-1]

    def get_json_data(self, packet_type, version=None):
        if version is None:
            version = self._last_version

        try:
            packet_types = self._jdata[str(version)]
        except KeyError:
            raise UnknownUDPVersionError

        try:
            packet_type = packet_types[str(packet_type)]
        except KeyError:
            raise UnknownUDPPacketTypeError

        if len([f["dtype"] for f in packet_type if f["dtype"][0] != "<"]) > 0:
            raise ValueError("Endianess not supported")
        return packet_type

    def get_field_names(self, packet_type, version=None):
        return [f["field_name"] for f in self.get_json_data(packet_type, version=version)]

    def get_numpy_field_dtypes(self, packet_type, version=None):
        return [f["dtype"] for f in self.get_json_data(packet_type, version=version)]

    def get_struct_format(self, packet_type, version):
        conversion_list = [
            self.STRUCT_CONVERSION[f]
            for f in self.get_numpy_field_dtypes(packet_type, version=version)
        ]
        return "<" + "".join(conversion_list)

    def unpack_data(self, data):
        version = data[0]
        packet_type = data[1]
        return struct.unpack(self.get_struct_format(packet_type, version=version), data)

    def unpack_data_dict(self, data):
        version = data[0]
        packet_type = data[1]
        struct_format = self.get_struct_format(packet_type, version=version)
        return dict(zip(self.get_field_names(packet_type), struct.unpack(struct_format, data)))

    def np_array_from_file(self, abspath):
        open_ = gzip.open if abspath.endswith(".gz") else open
        with open_(abspath, "rb") as bin_file:
            try:
                version, packet_type = bin_file.peek(2)[:2]
            except ValueError:
                return None
            dtype = list(
                zip(
                    self.get_field_names(packet_type, version=version),
                    self.get_numpy_field_dtypes(packet_type, version=version),
                )
            )
            try:
                data = bin_file.read()
            except EOFError:
                return None
            row_len = struct.calcsize(self.get_struct_format(packet_type, version=version))
            # Calculate how many full rows are in the file
            n_rows = int(len(data) / row_len)
            # Truncate after last full row if there is corrupt data
            data = data[: (n_rows * row_len)]
            return np.frombuffer(data, dtype=dtype)

    def pack_data(self, data):
        version = data[0]
        packet_type = data[1]
        if version is None:
            data = list(data)
            data[0] = int(self._last_version)
        return struct.pack(self.get_struct_format(packet_type, version=version), *data)



#!/usr/bin/env python3
import socket

import pytest
from blueye.legacyprotocol.exceptions import (
    MismatchedReply,
    NoConnectionToDrone,
    ResponseTimeout,
)


@pytest.fixture
def mocked_socket(mocker):
    return mocker.patch("socket.socket", autospec=True).return_value


@pytest.fixture
def mocked_struct(mocker):
    mocker.patch("struct.unpack", autospec=True)


@pytest.fixture
def mocked_logger(mocker):
    return mocker.patch("logging.Logger", autospec=True).return_value


@pytest.fixture
def tcp_client_v1(mocked_socket, mocked_logger):
    from blueye.legacyprotocol import TcpClient

    tc = TcpClient(protocol_version=1, autoConnect=False)
    tc.connect()
    tc.logger = mocked_logger
    yield tc


@pytest.fixture
def tcp_client_v2(mocked_socket, mocked_logger):
    from blueye.legacyprotocol import TcpClient

    tc = TcpClient(protocol_version=2, autoConnect=False)
    tc.connect()
    tc.logger = mocked_logger
    yield tc


@pytest.fixture(params=["tcp_client_v1", "tcp_client_v2"])
def tcp_client(request):
    """
    TCP client fixture that parametrizes protocol version 1 and 2, tests that use this
    fixture will be run with both protocol version 1 and 2
    """
    return request.getfixturevalue(request.param)


@pytest.mark.parametrize(
    "function_name, expected_message",
    [
        ("auto_heading_on", b"h"),
        ("auto_heading_off", b"H"),
        ("auto_depth_on", b"d"),
        ("auto_depth_off", b"D"),
        ("start_recording", b"r"),
        ("stop_recording", b"R"),
        ("start_compass_calibration", b"i"),
        ("cancel_compass_calibration", b"I"),
        ("save_compass_calibration", b"c"),
    ],
)
def test_commands_produce_correct_message(tcp_client, function_name, expected_message):
    func = getattr(tcp_client, function_name)
    func()
    tcp_client._sock.send.assert_called_with(expected_message)


def test_take_still_picture_produces_correct_message(tcp_client_v2):
    tcp_client_v2.take_still_picture()
    tcp_client_v2._sock.send.assert_called_with(b"s")


@pytest.mark.parametrize(
    "top_lights, bottom_lights, expected_message",
    [(0, 0, b"l\x00\x00"), (50, 50, b"l\x32\x32"), (255, 255, b"l\xff\xff")],
)
def test_set_light_command_produces_correct_message(
    tcp_client, top_lights, bottom_lights, expected_message
):
    tcp_client.set_lights(top_lights, bottom_lights)
    tcp_client._sock.send.assert_called_with(expected_message)


def test_ping_command_produces_correct_message(tcp_client, mocked_socket):
    correct_reply = b"P"
    mocked_socket.recv.return_value = correct_reply
    tcp_client.ping()
    tcp_client._sock.send.assert_called_with(b"p")


def test_exception_raised_on_wrong_reply(tcp_client, mocked_socket):
    mocked_socket.recv.return_value = "wrong reply"
    with pytest.raises(MismatchedReply):
        tcp_client.ping()


@pytest.mark.parametrize(
    "surge_input, sway_input, heave_input, yaw_input, slow_input, boost_input, tilt_speed_input, expected_message",
    [
        (
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            b"J\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
        ),
        (
            1,
            1,
            1,
            1,
            1,
            1,
            1,
            b"J\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?",
        ),
        (
            0.555,
            0.555,
            0.555,
            0.555,
            0.555,
            0.555,
            0.555,
            b"J{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?",
        ),
    ],
)
def test_motion_tilt_command_produces_correct_message(
    tcp_client_v2,
    surge_input,
    sway_input,
    heave_input,
    yaw_input,
    slow_input,
    boost_input,
    tilt_speed_input,
    expected_message,
):
    tcp_client_v2.motion_input_tilt(
        surge_input, sway_input, heave_input, yaw_input, slow_input, boost_input, tilt_speed_input
    )
    tcp_client_v2._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize(
    "out_of_range_input_arguments",
    [
        [0, 0, 0, 0, 0, -1, 0],
        [0, 0, 0, 0, 100, 0, 0],
        [100, 0, 0, 0, 0, 0, 0],
        [100, 0, 0, 0, 100, 0, 0],
        [0, 0, 0, 0, 0, 0, -100],
        [0, 0, 0, 0, 0, 0, 100],
    ],
)
def test_motion_tilt_command_raises_exception_when_input_out_of_range(
    tcp_client_v2, out_of_range_input_arguments
):
    with pytest.raises(ValueError):
        tcp_client_v2.motion_input_tilt(*out_of_range_input_arguments)


@pytest.mark.parametrize(
    "in_range_input_arguments",
    [[0, 0, 0, 0, 0, 0, 0], [1, 1, 1, 1, 1, 1, 1], [-1, -1, -1, -1, 0, 0, -1]],
)
def test_motion_tilt_commmand_does_not_raise_exception_when_input_arguments_are_in_range(
    tcp_client_v2, in_range_input_arguments
):
    tcp_client_v2.motion_input_tilt(*in_range_input_arguments)


@pytest.mark.parametrize(
    "surge_input, sway_input, heave_input, yaw_input, slow_input, boost_input, expected_message",
    [
        (
            0,
            0,
            0,
            0,
            0,
            0,
            b"j\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
        ),
        (
            1,
            1,
            1,
            1,
            1,
            1,
            b"j\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?\x00\x00\x80?",
        ),
        (
            0.555,
            0.555,
            0.555,
            0.555,
            0.555,
            0.555,
            b"j{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?{\x14\x0e?",
        ),
    ],
)
def test_motion_command_produces_correct_message(
    tcp_client,
    surge_input,
    sway_input,
    heave_input,
    yaw_input,
    slow_input,
    boost_input,
    expected_message,
):
    tcp_client.motion_input(
        surge_input, sway_input, heave_input, yaw_input, slow_input, boost_input
    )
    tcp_client._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize(
    "out_of_range_input_arguments",
    [
        [0, 0, 0, 0, 0, -1],
        [0, 0, 0, 0, 100, 0],
        [100, 0, 0, 0, 0, 0],
        [100, 0, 0, 0, 100, 0],
    ],
)
def test_motion_command_raises_exception_when_input_out_of_range(
    tcp_client, out_of_range_input_arguments
):
    with pytest.raises(ValueError):
        tcp_client.motion_input(*out_of_range_input_arguments)


@pytest.mark.parametrize(
    "in_range_input_arguments",
    [[0, 0, 0, 0, 0, 0], [1, 1, 1, 1, 1, 1], [-1, -1, -1, -1, 0, 0]],
)
def test_motion_commmand_does_not_raise_exception_when_input_arguments_are_in_range(
    tcp_client, in_range_input_arguments
):
    tcp_client.motion_input(*in_range_input_arguments)


@pytest.mark.parametrize("out_of_range_input_arguments", [[-10, -10], [300, 300]])
def test_light_command_raises_exception_when_input_out_of_range(
    tcp_client, out_of_range_input_arguments
):
    with pytest.raises(ValueError):
        tcp_client.set_lights(*out_of_range_input_arguments)


@pytest.mark.parametrize("in_range_input_arguments", [[0, 0], [255, 255]])
def test_light_command_does_not_raise_exception_when_input_arguments_are_in_range(
    tcp_client, in_range_input_arguments
):
    tcp_client.set_lights(*in_range_input_arguments)


def test_receive_msg_warns_on_timeout(tcp_client):
    tcp_client._sock.recv.side_effect = socket.timeout
    with pytest.raises(ResponseTimeout):
        tcp_client.receive_msg()
    tcp_client.logger.warning.assert_called_once()


@pytest.mark.parametrize(
    "latitude, longitude, expected_message",
    [
        (0, 0, b"g\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"),
        (10, 10, b"g\x00\x00\x00\x00\x00\x00$@\x00\x00\x00\x00\x00\x00$@"),
    ],
)
def test_user_geo_location_produces_correct_message(
    tcp_client, latitude, longitude, expected_message
):
    tcp_client.user_geo_location(latitude, longitude)
    tcp_client._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize(
    "connection_duration, expected_message", [(1000, b"w\xe8\x03"), (0, b"w\x00\x00")]
)
def test_watchdog_produces_correct_message(tcp_client, connection_duration, expected_message):
    tcp_client.watchdog(connection_duration)
    tcp_client._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize("direction, expected_message", [(-1, b"a\xff\xff"), (1, b"a\x01\x00")])
def test_auto_depth_step_produces_correct_message(tcp_client_v2, direction, expected_message):
    tcp_client_v2.auto_depth_step(direction)
    tcp_client_v2._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize("direction, expected_message", [(-1, b"A\xff\xff"), (1, b"A\x01\x00")])
def test_auto_heading_step_produces_correct_message(tcp_client_v2, direction, expected_message):
    tcp_client_v2.auto_heading_step(direction)
    tcp_client_v2._sock.send.assert_called_with(expected_message)


def test_connect_logs_error_on_failure(tcp_client):
    tcp_client._sock.connect.side_effect = socket.timeout
    with pytest.raises(NoConnectionToDrone):
        tcp_client.connect()
    tcp_client.logger.error.assert_called_once()


def test_connect_retries_on_failure(tcp_client, mocker):
    tcp_client._sock.connect.side_effect = socket.timeout
    # Resetting call count as connect is called when creating the mock
    tcp_client._sock.connect.call_count = 0
    with pytest.raises(NoConnectionToDrone):
        tcp_client.connect(max_retries=3)
    assert tcp_client._sock.connect.call_count == 4


@pytest.mark.parametrize(
    "system_time, expected_message",
    [(1500, b"t\xdc\x05\x00\x00"), (0, b"t\x00\x00\x00\x00")],
)
def test_set_system_time_produces_correct_message(
    tcp_client, mocked_socket, system_time, expected_message
):
    correct_reply = b"a"
    mocked_socket.recv.return_value = correct_reply
    tcp_client.set_system_time(system_time)
    tcp_client._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize(
    "camera_setting_function, parameter_value, expected_message",
    [
        ("set_camera_exposure", 1000, b"ve\xe8\x03\x00\x00"),
        ("set_camera_whitebalance", 1000, b"vw\xe8\x03\x00\x00"),
        ("set_camera_hue", 1000, b"vh\xe8\x03\x00\x00"),
        ("set_camera_bitrate", 1000, b"vb\xe8\x03\x00\x00"),
    ],
)
def test_camera_setting_functions_v1_produces_correct_messages(
    tcp_client_v1,
    mocked_socket,
    camera_setting_function,
    parameter_value,
    expected_message,
):
    correct_reply = b"a"
    mocked_socket.recv.return_value = correct_reply
    func = getattr(tcp_client_v1, camera_setting_function)
    func(parameter_value)
    tcp_client_v1._sock.send.assert_called_with(expected_message)


@pytest.mark.parametrize(
    "camera_setting_function, parameter_value, expected_message",
    [
        ("set_camera_framerate", 1000, b"vf\xe8\x03\x00\x00"),
        ("set_camera_resolution", 1000, b"vr\xe8\x03\x00\x00"),
    ],
)
def test_camera_setting_functions_v2_produce_correct_messages(
    tcp_client_v2,
    mocked_socket,
    camera_setting_function,
    parameter_value,
    expected_message,
):
    """
    Test that runs the camera setting functions introduced in the second tcp protocol version
    """
    correct_reply = b"a"
    mocked_socket.recv.return_value = correct_reply
    func = getattr(tcp_client_v2, camera_setting_function)
    func(parameter_value)
    tcp_client_v2._sock.send.assert_called_with(expected_message)


def test_get_camera_parameters_produces_correct_message(tcp_client, mocked_socket, mocked_struct):
    tcp_client.get_camera_parameters()
    tcp_client._sock.send.assert_called_with(b"Va")


def test_not_specifying_protocol_results_in_latest_protocol(mocked_socket, mocked_logger):
    from blueye.legacyprotocol import TcpClient

    tc = TcpClient(autoConnect=False)
    assert tc.protocol_version == 2


@pytest.mark.parametrize("density, expected_message", [(0, b"W\x00\x00"), (1000, b"W\xe8\x03")])
def test_set_water_density_produces_correct_message(
    tcp_client_v2, mocked_socket, density, expected_message
):
    tcp_client_v2.set_water_density(density)
    tcp_client_v2._sock.send.assert_called_with(expected_message)
    tcp_client_v2._sock.send.assert_called_with(expected_message)
    tcp_client_v2._sock.send.assert_called_with(expected_message)



class TCPClientException(Exception):
    """Generic exception related to the TCPClient"""


class SocketNotConnected(TCPClientException):
    """Raised when trying to use a socket that is not connected"""


class ResponseTimeout(TCPClientException):
    """Raised when the socket timed out waiting for response to a message"""


class MismatchedReply(TCPClientException, ValueError):
    """Raised when the TCP Client receives an unexpected reply from the drone"""

    def __init__(self, expected_reply, actual_reply):
        self.expected_reply = expected_reply
        self.actual_reply = actual_reply
        TCPClientException.__init__(self, ("Unexpected reply from drone, expected: " +
                                           f"{expected_reply}, but got: {actual_reply}"))


class NoConnectionToDrone(TCPClientException):
    """Raised if the TCP Client is unable to establish a connection to the drone"""

    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        TCPClientException.__init__(self, f"Unable to connect to drone at {ip}:{port}")


class UnknownUDPVersionError(Exception):
    """Raised if the UDP parser does not know the version of the UDP packet"""


class UnknownUDPPacketTypeError(Exception):
    """Raised if the UDP parser does not know the type of the UDP packet"""



#!/usr/bin/env python3
import logging
import socket
import threading

from .exceptions import (MismatchedReply, NoConnectionToDrone,
                                        ResponseTimeout, SocketNotConnected)


class TcpClientBase(threading.Thread):
    """A base level TcpClient for communicating with the Blueye Pioneer

    TcpClientBase should not be instantiated on its own. Instead protocol version classes inherit
    the functionality from TcpClientBase, and the combined functionality of a protocol version
    class and TcpClientBase are exposed in the top level class TcpClient.

    TcpClientBase -> TcpclientV1(or other protocol versions) -> TcpClient.

    Example of how to connect to a drone, and toggle the lights
    Example:
    >>> import time
    >>> import blueye.protocol
    >>> tc = blueye.protocol.TcpClient(maxConnectRetries=3)
    >>> tc.set_lights(10, 10)
    >>> time.sleep(3)
    >>> tc.set_lights(0, 0)
    """

    def __init__(
        self,
        port=2011,
        ip="192.168.1.101",
        maxConnectRetries=0,
        autoConnect=True,
        logger=None,
    ):
        threading.Thread.__init__(self)
        self._ip = ip
        self._port = port

        if logger is None:
            # If no logger has been passed we'll use the module logger
            self.logger = logging.getLogger(__name__)
        else:
            self.logger = logger

        self._sock = None
        self.socket_lock = threading.Lock()

        self._exit_flag = threading.Event()
        self.daemon = True

        if autoConnect is True:
            self.connect(maxConnectRetries)
            self.start()

    def run(self):
        """Keep the TCP connection to the drone alive by sending watchdog commands.
        """
        i = 0
        WATCHDOG_DELAY = 1
        while not self._exit_flag.wait(timeout=WATCHDOG_DELAY):
            self.watchdog(i)
            i += 1

    def connect(self, max_retries=0):
        """Connect the TcpClient to the drone

        Args:
            max_retries(int): Make max_retries attempt at connecting to the drone before failing
        """
        attempts = 0
        while attempts <= max_retries:
            try:
                # Creating a new socket for each attempt, as a broken socket should not be reused.
                self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self._sock.settimeout(1.0)
                self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                self._sock.connect((self._ip, self._port))
            except socket.timeout:
                self._sock.close()
                attempts += 1
                continue
            break
        else:
            self.logger.error(
                f"Attempted {attempts} time{'' if attempts == 1 else 's'}, "
                "but was unable to connect to drone.")
            raise NoConnectionToDrone(self._ip, self._port)

    def stop(self):
        """Stop the watchdog thread started by run()
        """
        self._exit_flag.set()

    def send_msg(self, msg):
        """Send a binary message to the drone

        Args:
            msg (bytes): The message to be sent
        """
        if self._sock is None:
            raise SocketNotConnected
        self.logger.debug(f"Sent message: {msg}")
        self._sock.send(msg)

    def receive_msg(self, size=1):
        """Receive a binary message from the drone

        Args:
           receive_size (int): The expected byte count of the reply

        Returns:
            reply (bytes): The reply from the drone
        """
        try:
            reply = self._sock.recv(size)
            self.logger.debug(f"Received message: {reply}")
            return reply
        except socket.timeout as e:
            self.logger.warning("Timed out while waiting for message reply")
            raise ResponseTimeout from e

    def send_and_receive(self, msg, expects_reply=True, receive_size=1):
        """Send a binary message and return the drones reply

        Messages that have a reply in the form of data or a ack from the drone
        should be sent using this method to ensure messages send are correctly
        matched up to their replies

        Args:
            msg (bytes): The message to be sent
            expects_reply (bool): True if the message has a expected reply
            receive_size (int): The byte count of the expected reply

        Returns:
            reply (bytes): The reply from the drone
        """
        with self.socket_lock:
            self.send_msg(msg)
            if expects_reply:
                reply = self.receive_msg(receive_size)
                return reply
            return 0

    def check_reply(self, reply, expected_reply):
        """ Check that reply matched expected reply, raise exception if not

        Args:
            reply (str): The reply from the drone
            expected_reply (str): The expected reply from the drone
        """
        if not reply == expected_reply:
            raise MismatchedReply(expected_reply, reply)


if __name__ == "__main__":
    import doctest
    doctest.testmod()  # test examples in docstrings if module run as main



#!/usr/bin/env python3
import gzip
import json
import struct
import tempfile
import unittest

from blueye.legacyprotocol import AppProtocol
from blueye.legacyprotocol.exceptions import (
    UnknownUDPPacketTypeError,
    UnknownUDPVersionError,
)

fake_data = json.loads(
    """
[
    {
        "version": "1",
        "messages": [
            {
                "name": "telemetry",
                "message_type": "1",
                "fields":[
                    {"description": "", "dtype": "<u1", "field_name": "u1-1-v", "unit": ""},
                    {"description": "", "dtype": "<u1", "field_name": "u1-1-t", "unit": ""},
                    {"description": "", "dtype": "<i1", "field_name": "i1-1", "unit": ""},
                    {"description": "", "dtype": "<u2", "field_name": "u2-1", "unit": ""},
                    {"description": "", "dtype": "<i2", "field_name": "i2-1", "unit": ""},
                    {"description": "", "dtype": "<u4", "field_name": "u4-1", "unit": ""},
                    {"description": "", "dtype": "<i4", "field_name": "i4-1", "unit": ""},
                    {"description": "", "dtype": "<u8", "field_name": "u8-1", "unit": ""},
                    {"description": "", "dtype": "<i8", "field_name": "i8-1", "unit": ""},
                    {"description": "", "dtype": "<f4", "field_name": "f4-1", "unit": ""},
                    {"description": "", "dtype": "<f8", "field_name": "f8-1", "unit": ""}
                ]
            }
        ]
    },
    {
        "version": "2",
        "messages": [
            {
                "name": "telemetry",
                "message_type": "1",
                "fields":[
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-v", "unit": ""},
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-t", "unit": ""},
                    {"description": "", "dtype": "<i1", "field_name": "i1-2", "unit": ""},
                    {"description": "", "dtype": "<u2", "field_name": "u2-2", "unit": ""},
                    {"description": "", "dtype": "<i2", "field_name": "i2-2", "unit": ""},
                    {"description": "", "dtype": "<u4", "field_name": "u4-2", "unit": ""},
                    {"description": "", "dtype": "<i4", "field_name": "i4-2", "unit": ""},
                    {"description": "", "dtype": "<u8", "field_name": "u8-2", "unit": ""},
                    {"description": "", "dtype": "<i8", "field_name": "i8-2", "unit": ""},
                    {"description": "", "dtype": "<f4", "field_name": "f4-2", "unit": ""},
                    {"description": "", "dtype": "<f8", "field_name": "f8-2-a", "unit": ""},
                    {"description": "", "dtype": "<f8", "field_name": "f8-2-b", "unit": ""}
                ]
            },
            {
                "name": "telemetry",
                "message_type": "2",
                "fields":[
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-v", "unit": ""},
                    {"description": "", "dtype": "<u1", "field_name": "u1-2-t", "unit": ""},
                    {"description": "", "dtype": "<i1", "field_name": "i1-2", "unit": ""}
                ]
            }
        ]
    }
]
"""
)

fake_data_endianess = json.loads(
    """
[
    {
        "version": "1",
        "messages": [
            {
                "name": "telemetry",
                "message_type": "1",
                "fields":[
                   {"description": "", "dtype": "<u1", "field_name": "u1-1", "unit": ""},
                    {"description": "", "dtype": ">i1", "field_name": "i1-1", "unit": ""},
                    {"description": "", "dtype": "<u2", "field_name": "u2-1", "unit": ""}
                ]
            }
        ]
    }
]"""
)


class TestAppProtocol(unittest.TestCase):
    def setUp(self):
        pass

    def test_struct(self):
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.get_struct_format(1, 1), "<BBbHhIiQqfd")

    def test_struct_unpack_v1_t1(self):
        data = (1, 1, 2, 3, 4, 5, 6, 7, 8, 9.0, 10.0)
        data_packet = struct.pack("<BBbHhIiQqfd", *data)
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.unpack_data(data_packet), data)

    def test_struct_unpack_v2_t1(self):
        data = (2, 1, 2, 3, 4, 5, 6, 7, 8, 9.0, 10.0, 11.0)
        data_packet = struct.pack("<BBbHhIiQqfdd", *data)
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.unpack_data(data_packet), data)

    def test_struct_unpack_v2_t2(self):
        data = (2, 2, 2)
        data_packet = struct.pack("<BBb", *data)
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.unpack_data(data_packet), data)

    def test_struct_unpack_data_dict_v2_t2(self):
        data = (2, 2, 2)
        data_dict = {"u1-2-v": 2, "u1-2-t": 2, "i1-2": 2}
        data_packet = struct.pack("<BBb", *data)
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.unpack_data_dict(data_packet), data_dict)

    def test_struct_pack_v2_t2(self):
        data = (2, 2, 2)
        data_packet = struct.pack("<BBb", *data)
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.pack_data(data), data_packet)

    def test_struct_pack_None_t2(self):
        data_none = (None, 2, 2)
        data = (2, 2, 2)
        data_packet = struct.pack("<BBb", *data)
        ap = AppProtocol(fake_data)
        self.assertEqual(ap.pack_data(data_none), data_packet)

    def test_protocol_version_none(self):
        ap = AppProtocol(fake_data)
        self.assertEqual(ap._last_version, "2")

    def test_wrong_endianess(self):
        ap = AppProtocol(fake_data_endianess)
        self.assertRaises(ValueError, ap.get_json_data, 1, 1)

    def test_protocol_version_exception(self):
        ap = AppProtocol(fake_data)
        self.assertRaises(UnknownUDPVersionError, ap.get_json_data, 1, 0)

    def test_protocol_packet_type_exception(self):
        ap = AppProtocol(fake_data)
        self.assertRaises(UnknownUDPPacketTypeError, ap.get_json_data, 0, 1)

    def test_nptypes(self):
        ap = AppProtocol(fake_data)
        self.assertEqual(
            ap.get_numpy_field_dtypes(1, 1),
            [
                "<u1",
                "<u1",
                "<i1",
                "<u2",
                "<i2",
                "<u4",
                "<i4",
                "<u8",
                "<i8",
                "<f4",
                "<f8",
            ],
        )

    def test_field_names_v1_t1(self):
        ap = AppProtocol(fake_data)
        self.assertEqual(
            ap.get_field_names(1, 1),
            [
                "u1-1-v",
                "u1-1-t",
                "i1-1",
                "u2-1",
                "i2-1",
                "u4-1",
                "i4-1",
                "u8-1",
                "i8-1",
                "f4-1",
                "f8-1",
            ],
        )

    def test_field_names_v2_t1(self):
        ap = AppProtocol(fake_data)
        self.assertEqual(
            ap.get_field_names(1, 2),
            [
                "u1-2-v",
                "u1-2-t",
                "i1-2",
                "u2-2",
                "i2-2",
                "u4-2",
                "i4-2",
                "u8-2",
                "i8-2",
                "f4-2",
                "f8-2-a",
                "f8-2-b",
            ],
        )

    def test_field_names_None_t1(self):
        ap = AppProtocol(fake_data)
        self.assertEqual(
            ap.get_field_names(1),
            [
                "u1-2-v",
                "u1-2-t",
                "i1-2",
                "u2-2",
                "i2-2",
                "u4-2",
                "i4-2",
                "u8-2",
                "i8-2",
                "f4-2",
                "f8-2-a",
                "f8-2-b",
            ],
        )

    def test_np_array_from_file(self):
        data = (
            2,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9.0,
            10.0,
            11.0,
            2,
            1,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19.0,
            20.0,
            21.0,
        )
        data_packet = struct.pack("<BBbHhIiQqfddBBbHhIiQqfdd", *data)
        ap = AppProtocol(fake_data)
        tmp = tempfile.mkstemp()[1]
        with open(tmp, "wb") as f:
            f.write(data_packet)
        np_data = ap.np_array_from_file(tmp)
        self.assertEqual(np_data["i8-2"].tolist(), [8, 18])
        self.assertEqual([data[:12], data[12:]], np_data.tolist())

    def test_np_array_from_gz_file(self):
        data = (
            2,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9.0,
            10.0,
            11.0,
            2,
            1,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19.0,
            20.0,
            21.0,
        )
        data_packet = struct.pack("<BBbHhIiQqfddBBbHhIiQqfdd", *data)
        ap = AppProtocol(fake_data)
        tmp = tempfile.mkstemp(suffix=".gz")[1]
        with gzip.open(tmp, "wb") as f:
            f.write(data_packet)
        np_data = ap.np_array_from_file(tmp)
        self.assertEqual(np_data["i8-2"].tolist(), [8, 18])
        self.assertEqual([data[:12], data[12:]], np_data.tolist())

    def test_np_array_from_empty_file(self):
        ap = AppProtocol(fake_data)
        tmp = tempfile.mkstemp()[1]
        with open(tmp, "wb"):
            pass
        np_data = ap.np_array_from_file(tmp)
        self.assertIsNone(np_data)


if __name__ == "__main__":
    unittest.main()
